/**
 * (C) 2011-2012 Alibaba Group Holding Limited.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 */
/**
 * Copyright 2007 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.taobao.common.tedis.support.lucene.analysis.xanalyzer;

import java.io.Reader;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.TokenStream;

import com.taobao.common.tedis.support.lucene.analysis.xanalyzer.collector.QueryTokenCollector;
import com.taobao.common.tedis.support.lucene.analysis.xanalyzer.collector.WriterTokenCollector;
import com.taobao.common.tedis.support.paoding.Knife;
import com.taobao.common.tedis.support.paoding.Paoding;
import com.taobao.common.tedis.support.paoding.cjk.CJKKnife;

/**
 * XAnalyzerǻڡҶţܵLuceneǡҶţܶLucene
 * <p>
 *
 * XAnalyzer̰߳ȫģʹͬһXAnalyzerʵǿеġ<br>
 * XAnalyzerǿɸõģƼͬһXAnalyzerʵ
 * <p>
 *
 * ҪرӦͨ캯knife(setter)ԶƵKnifeʵ
 * <p>
 *
 * @author Zhiliang Wang [qieqie.wang@gmail.com]
 *
 * @see XWriterAnalyzer
 * @see XQueryAnalyzer
 *
 * @see XTokenizer
 * @see Knife
 * @see Paoding
 * @see CJKKnife
 * @see TokenCollector
 *
 * @since 1.0
 *
 */
public class XAnalyzer extends Analyzer {

    // -------------------------------------------------

    /**
     * ģʽڽʱʹãܹʹÿܵĴｨ
     */
    public static final int WRITER_MODE = 1;

    /**
     * ģʽûʱʹãʹûĽƥ
     */
    public static final int QUERY_MODE = 2;

    // -------------------------------------------------
    /**
     * XTokenizerṩֽıַ
     *
     * @see XTokenizer#next()
     *
     */
    private Knife knife;

    /**
     * @see #WRITER_MODE
     * @see #QUERY_MODE
     */
    private int mode = WRITER_MODE;

    // -------------------------------------------------

    public XAnalyzer() {
    }

    public XAnalyzer(Knife knife) {
        this.knife = knife;
    }

    // -------------------------------------------------

    public Knife getKnife() {
        return knife;
    }

    public void setKnife(Knife knife) {
        this.knife = knife;
    }

    public int getMode() {
        return mode;
    }

    /**
     * ÷ģʽдģʽ(WRITER_MODE)ģʽ(QUERY_MODE)һ֡ĬΪдģʽ
     * <p>
     * WRITER_MODEڽʱʹãܹʹÿܵĴｨ<br>
     * QUERY_MODEûʱʹãʹûĽƥ
     *
     * @param mode
     */
    public void setMode(int mode) {
        this.mode = mode;
    }

    // -------------------------------------------------

    @Override
    public TokenStream tokenStream(String fieldName, Reader reader) {
        if (knife == null) {
            throw new NullPointerException("knife should be set before token");
        }
        // XTokenizerTokenStreamʵ֣ʹknifereaderı
        return new XTokenizer(reader, knife, createTokenCollector());
    }

    protected TokenCollector createTokenCollector() {
        switch (mode) {
        case WRITER_MODE:
            return new WriterTokenCollector();
        case QUERY_MODE:
            return new QueryTokenCollector();
        default:
            throw new IllegalArgumentException("wrong mode");
        }
    }

}
